﻿using System;
using System.ComponentModel;
using System.Windows.Input;
using Hitsuji.Framework.Models;

namespace Hitsuji.Framework.Commands
{
    public delegate bool CanExecuteDelegate();
    public delegate void executeDelegate();

    public class PropertyChangedCommand : ICommand
    {

        protected CommandBindableModel CommandBindableModel
        {
            get;
            set;
        }


        #region ▼イベント（処理）の初期設定

        protected void initEventTemplate()
        {
            CommandBindableModel.PropertyChanged += eventTemplate;
        }
        protected void initMethodTemplate()
        {
            CommandBindableModel.PropertyChanged += methodTemplate;
        }

        #endregion ▲イベント（処理）の初期設定


        #region ▼イベント（処理）の設定登録・解除

        public void addEventTemplate()
        {
            CommandBindableModel.PropertyChanged += eventTemplate;
        }
        public void removeEventTemplate()
        {
            CommandBindableModel.PropertyChanged -= eventTemplate;
        }

        #endregion ▲イベント（処理）の設定登録・解除


        #region ▼イベント（処理）のテンプレート

        private void methodTemplate(object sender, PropertyChangedEventArgs e)
        {
            if (CanExecute(null))
                Execute(null);
        }

        private void eventTemplate(object sender, PropertyChangedEventArgs e)
        {
            if (CanExecuteChanged != null)
                CanExecuteChanged(this, EventArgs.Empty);
        }

        #endregion ▲イベント（処理）のテンプレート


        #region ICommand メンバ

        public bool CanExecute(object parameter)
        {
            if (CommandBindableModel.canExeDelegate_ != null)
            {
                return CommandBindableModel.canExeDelegate_();
            }
            else
            {
                return false;
            }
        }

        public event EventHandler CanExecuteChanged;

        public void Execute(object parameter)
        {
            if (CommandBindableModel.exeDelegate_ != null)
            {
                CommandBindableModel.exeDelegate_();
            }
            else
            {
                return;
            }
        }

        #endregion
    }
}
