﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;

namespace Hitsuji.Framework.Data
{
    /// <summary>
    /// System.Data.DataSet にCSVファイルの読込・書込み機能を追加したもの
    /// </summary>
    public class HDataSet : DataSet
    {

        #region ▼CSVファイル読込

        //ＣＳＶファイル読込
        public void readCsv(String filePath,
                            char separator,
                            String encoding)
        {
            try
            {
                var columnMaxSize = 1;
                var recordList = new List<String[]>();
                var allLinesInTextFile = System.IO.File.ReadAllLines(filePath);
                foreach (String oneLine in allLinesInTextFile)
                {
                    String[] columnArray = oneLine.Split(separator);
                    //カラム数の最大値を取得
                    if (columnMaxSize < columnArray.Length)
                        columnMaxSize = columnArray.Length;
                    //行をリストに追加
                    recordList.Add(columnArray);
                }
                
                var targetTableName = this.Tables.Count.ToString();
                this.Tables.Add(new DataTable(targetTableName));
                this.buildColumn(targetTableName, columnMaxSize);
                this.buildFields(recordList, targetTableName, columnMaxSize);
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        //カラムを設定
        protected void buildColumn(String targetTableName,
                                   int columnMaxSize)
        {
            try
            {
                for (int count = 0; count < columnMaxSize; count++)
                {
                    this.Tables[targetTableName].Columns.Add(new DataColumn(count.ToString()));
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }
        //フィールド設定
        protected void buildFields(List<String[]> records,
                                   String targetTableName,
                                   int columnMaxSize)
        {
            try
            {
                foreach (String[] line in records)
                {
                    var myRow = this.Tables[targetTableName].NewRow();
                    for (int columnIndex = 0; columnIndex < columnMaxSize; columnIndex++)
                    {
                        myRow[columnIndex] = line[columnIndex];
                    }
                    this.Tables[targetTableName].Rows.Add(myRow);
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        #endregion ▲CSVファイル読込


        #region ▼CSVファイル書き込み

        //ＣＳＶファイル書き込み
        public void writeCsv(String filePath,
                             String tableNameHavingData,
                             char separator,
                             bool columnsWrite)
        {
            try
            {
                var table = this.Tables[tableNameHavingData];
                List<String> lines = new List<String>();
                foreach (DataRow row in this.Tables[tableNameHavingData].Rows)
                {
                    var dataArray = row.ItemArray;
                    StringBuilder line = new StringBuilder();
                    foreach (object data in dataArray)
                    {
                        line.Append(data.ToString());
                        line.Append(separator);
                    }
                    lines.Add(line.ToString());
                }
                System.IO.File.WriteAllLines(filePath, lines.ToArray<String>());
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        #endregion ▲CSVファイル書き込み

    }
}
