﻿using System;
using System.Collections.Generic;
using Hitsuji.Framework.ObserverPattern.Observers;

namespace Hitsuji.Framework.ObserverPattern.Subjects
{
    public abstract class ASubject
    {

        private Dictionary<String, IObserver> observerDictionary;
        
        
        public IObserver getObserver(string observerClassName)
        {
            if (observerDictionary.Count <= 0){ throw new NullReferenceException();}
            IObserver observer = null;
            foreach (KeyValuePair<String, IObserver> kvp in observerDictionary)
            {
                observer = kvp.Value;
            }

            if (observer == null) { throw new NullReferenceException(); }
            return observer;
        }


        public void addObserver(IObserver observer)
        {
            try
            {
                if (observerDictionary == null) { observerDictionary = new Dictionary<string, IObserver>(); }
                this.observerDictionary.Add(observer.GetType().Name, observer);
            }
            catch(Exception ex)
            {
                Console.Write(ex.ToString());
                throw ex; 
            }
        }


        public void removeObserver(IObserver observer)
        {
            this.observerDictionary.Remove(observer.GetType().Name);
        }


        public void notify(ASubject subject)
        {
            if (observerDictionary == null ||
                observerDictionary.Count <= 0) { return; }
            foreach (KeyValuePair<String, IObserver> kvp in observerDictionary)
            {
                kvp.Value.doMyRole(this);
            }
        }


        public abstract void doMyRole();
        
    }
}
