﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Printing;
using System.IO;
using System.Linq;
using System.Windows.Forms;
using System.Xml.Linq;
using Hitsuji.Nengajou.Models.DataBases;
using Hitsuji.Nengajou.Print.Models.ContentModels;
using Hitsuji.Nengajou.Print;

namespace Hitsuji.Nengajou.Print.Controls
{
    public class PrintReseiverLayoutControlClass
    {

        int ReseiverLayoutCount = 0;
        string textAll = null;
        XElement doc = null;
        IEnumerable<XElement> query;
        //XElement elem;
        XElement innerElem;
        int pageCount;
        int currentPageCount;
        string PrintPaperName = "ﾊｶﾞｷ 100 x 148 mm";

        ComboBox comboBoxFontSize;
        ComboBox comboBoxMargin;
        public void initComboBoxFontSizeItems(ComboBox comboBox)
        {
            this.comboBoxFontSize = comboBox;
            comboBox.Items.Add(4);
            comboBox.Items.Add(2);
            comboBox.Items.Add(0);
            comboBox.Items.Add(-2);
            comboBox.Items.Add(-4);
            comboBox.SelectedItem = 0;
        }

        public void initComboBoxMargineItems(ComboBox comboBox)
        {
            this.comboBoxMargin = comboBox;
            comboBox.Items.Add(10);
            comboBox.Items.Add(5);
            comboBox.Items.Add(0);
            comboBox.Items.Add(-5);
            comboBox.Items.Add(-10);
            comboBox.SelectedItem = -10;
        }


        public void readAndPreviewReseiverLayout(object sender, EventArgs e)
        {
            try
            {
                ReseiverLayoutCount = 0;

                PrintDocument printDocument = new PrintDocument();
                PrintPreviewDialog printPreviewDialog = new PrintPreviewDialog();
                printPreviewDialog.Document = printDocument;

                printDocument.DefaultPageSettings.PaperSize = getPaperSize(printDocument, PrintPaperName);
                
                textAll = File.ReadAllText(FilePathDataBase.getPrintoutReseiverLayoutFilePath());
                doc = XElement.Parse(textAll);

                query = from n in doc.Descendants(ReseiverLayoutInfoTable.PrintDataTableName)
                        //where n.Element("ENABLE").Value.StartsWith(enableParam.ToString())
                        select n;

                //ページ数
                pageCount = query.Count<XElement>();
                currentPageCount = 0;
                if (pageCount >= 1)
                {
                    printDocument.PrintPage += new PrintPageEventHandler(pd_PrintPageReseiverLayout);
                }

                //印刷プレビューダイアログを表示する
                printPreviewDialog.ShowDialog();
            }
            catch (Exception ex)
            {
                System.Windows.Forms.MessageBox.Show(ex.ToString());
                Console.WriteLine(ex.ToString());
            }
        }

        private void pd_PrintPageReseiverLayout(object sender, PrintPageEventArgs e)
        {
            try
            {
                var elm = query.ElementAt<XElement>(currentPageCount);
                var list = new List<LabelContentModel>();
                if (elm.HasElements)
                {
                    foreach (var inner in elm.Elements(ReseiverLayoutInfoTable.label))
                    {
                        innerElem = inner;
                        buildLabelModelInfo(inner, list);
                    }
                }

                foreach (var labelModel in list)
                {
                    StringFormat format = new StringFormat();
                    format.FormatFlags = StringFormatFlags.DirectionVertical;
                    e.Graphics.DrawString(labelModel.Value,
                                          new Font(labelModel.FontFamily, int.Parse(labelModel.FontSize) + (int)this.comboBoxFontSize.SelectedItem),
                                          Hitsuji.Nengajou.Print.Color.ColorExchangeClass.exchangeAsColor(labelModel.FontColor),
                                          float.Parse(labelModel.X) + (int)this.comboBoxMargin.SelectedItem,
                                          float.Parse(labelModel.Y) + (int)this.comboBoxMargin.SelectedItem,
                                          format);
                }

                if (currentPageCount != pageCount - 1)
                {
                    e.HasMorePages = true;
                    currentPageCount += 1;
                }
                else
                {
                    e.HasMorePages = false;
                    currentPageCount = 0;
                }
            }
            catch (Exception ex)
            {
                e.HasMorePages = false;
                System.Windows.Forms.MessageBox.Show("印刷処理時にエラーが発生しました。");
            }
        }

        //用紙サイズ名取得
        public PaperSize getPaperSize(PrintDocument pd, string partOfPaperName)
        {
            for (int index = 0; index < pd.PrinterSettings.PaperSizes.Count; index++)
            {
                if (pd.PrinterSettings.PaperSizes[index].PaperName.Contains(partOfPaperName) == true)
                {
                    return pd.PrinterSettings.PaperSizes[index];
                }
            }
            throw new Exception("用紙サイズ指定時にエラーが発生しました。プリンタ周りの設定を確認してみて下さい。");
        }

        protected void buildLabelModelInfo(XElement elm, List<LabelContentModel> modelList)
        {
            try
            {
                var model = new LabelContentModel();
                model.FontSize = elm.Element(ReseiverLayoutInfoTable.label_FontSize).Value;
                model.FontFamily = elm.Element(ReseiverLayoutInfoTable.label_FontFamily).Value;
                model.FontColor = elm.Element(ReseiverLayoutInfoTable.label_FontColor).Value;
                model.X = elm.Element(ReseiverLayoutInfoTable.label_X).Value;
                model.Y = elm.Element(ReseiverLayoutInfoTable.label_Y).Value;
                model.Width = elm.Element(ReseiverLayoutInfoTable.label_Width).Value;
                model.Height = elm.Element(ReseiverLayoutInfoTable.label_Height).Value;
                model.Value = elm.Element(ReseiverLayoutInfoTable.label_Value).Value;
                modelList.Add(model);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }
    }
}
