﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Linq;
using Hitsuji.Framework.ObserverPattern.Observers;
using Hitsuji.Framework.ObserverPattern.Subjects;
using Hitsuji.Nengajou.Models.Contents;
using Hitsuji.Nengajou.Models.DataBases;
using Hitsuji.Nengajou.ObserverPattern.Subjects;

namespace Hitsuji.Nengajou.ObserverPattern.Observers
{
    public class CreateReseiverLayoutForPrintObserver : IObserver
    {
        #region IObserver メンバ

        public void doMyRole(ASubject aSubject)
        {
            try
            {
                GetReseiverLayoutInfoForPrintoutSubject subject = aSubject as GetReseiverLayoutInfoForPrintoutSubject;
                var infoList = subject.models["infoList"] as List<IContentModel>;

                var layoutDictionary = new Dictionary<string,XElement>();
                foreach(var layout in subject.reseiverLayoutElements)
                {
                    layoutDictionary.Add(layout.Element(ReseiverLayoutInfoTable.Id).Value, layout);
                }

                var doc = new XDocument();
                var rootElem = new XElement(ReseiverLayoutInfoTable.PrintDataSetName);
                
                foreach (PrintReseiverLayoutInfoContentModel info in infoList)
                {
                    var layoutElem = new XElement(ReseiverLayoutInfoTable.PrintDataTableName,
                                            new XElement(ReseiverLayoutInfoTable.PrintPersonalId, info.ReseiverId),
                                            new XElement(ReseiverLayoutInfoTable.PrintReseiverLayoutId, info.ReseiverLayoutId)
                                            );
                    var layout = layoutDictionary[info.ReseiverLayoutId];
                    foreach (var labelElem in layout.Elements(ReseiverLayoutInfoTable.label))
                    {
                        createAndAddLabelElement(labelElem, layoutElem, info);
                    }
                    rootElem.Add(layoutElem);
                }
                doc.Add(rootElem);

                //XML書き込み
                var writer = new XmlTextWriter(FilePathDataBase.getPrintoutReseiverLayoutFilePath(), Encoding.UTF8);
                //var writer = new XmlTextWriter(FilePathDataBase.getDeskTop(), Encoding.UTF8);
                writer.Formatting = Formatting.Indented;
                doc.WriteTo(writer);
                writer.Close();
                writer = null;
                doc = null;
            
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }

        #endregion


        protected void createAndAddLabelElement(XElement labelElem, 
                                                XElement layoutElem, 
                                                PrintReseiverLayoutInfoContentModel info)
        {
            string labelName = labelElem.Element(ReseiverLayoutInfoTable.label_Name).Value;
            XElement valueElem;
            if (labelName.Equals("LBL_R_PRIM1"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.ReseiverPrim1);
            }
            else if (labelName.Equals("LBL_R_PRIM2"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.ReseiverPrim2);
            }
            else if (labelName.Equals("LBL_R_PRIM3"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.ReseiverPrim3);
            }
            else if (labelName.Equals("LBL_R_SECO1"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.ReseiverPost1);
            }
            else if (labelName.Equals("LBL_R_SECO2"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.ReseiverPost2);
            }
            else if (labelName.Equals("LBL_R_SECO3"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.ReseiverPost3);
            }
            else if (labelName.Equals("LBL_R_SECO4"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.ReseiverPost4);
            }
            else if (labelName.Equals("LBL_R_ADDR1"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.ReseiverAddress1);
            }
            else if (labelName.Equals("LBL_R_ADDR2"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.ReseiverAddress2);
            }
            else if (labelName.Equals("LBL_R_ADDR3"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.ReseiverAddress3);
            }
            else if (labelName.Equals("LBL_R_NAME1"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.ReseiverName1);
            }
            else if (labelName.Equals("LBL_R_NAME2"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.ReseiverName2);
            }
            else if (labelName.Equals("LBL_R_NAME3"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.ReseiverName3);
            }
            else if (labelName.Equals("LBL_R_NAME4"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.ReseiverName4);
            }
            else if (labelName.Equals("LBL_R_NPF1"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.ReseiverNPF1);
            }
            else if (labelName.Equals("LBL_R_NPF2"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.ReseiverNPF2);
            }
            else if (labelName.Equals("LBL_R_NPF3"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.ReseiverNPF3);
            }
            else if (labelName.Equals("LBL_R_NPF4"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.ReseiverNPF4);
            }

            else if (labelName.Equals("LBL_S_PRIM1"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.SenderPrim1);
            }
            else if (labelName.Equals("LBL_S_PRIM2"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.SenderPrim2);
            }
            else if (labelName.Equals("LBL_S_PRIM3"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.SenderPrim3);
            }
            else if (labelName.Equals("LBL_S_SECO1"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.SenderPost1);
            }
            else if (labelName.Equals("LBL_S_SECO2"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.SenderPost2);
            }
            else if (labelName.Equals("LBL_S_SECO3"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.SenderPost3);
            }
            else if (labelName.Equals("LBL_S_SECO4"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.SenderPost4);
            }
            else if (labelName.Equals("LBL_S_ADDR1"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.SenderAddress1);
            }
            else if (labelName.Equals("LBL_S_ADDR2"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.SenderAddress2);
            }
            else if (labelName.Equals("LBL_S_ADDR3"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.SenderAddress3);
            }
            else if (labelName.Equals("LBL_S_NAME1"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.SenderName1);
            }
            else if (labelName.Equals("LBL_S_NAME2"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.SenderName2);
            }
            else if (labelName.Equals("LBL_S_NAME3"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.SenderName3);
            }
            else if (labelName.Equals("LBL_S_NAME4"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.SenderName4);
            }
            else if (labelName.Equals("LBL_S_TEL"))
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, info.SenderTel);
            }
            
            else
            {
                valueElem = new XElement(ReseiverLayoutInfoTable.label_Value, "データに異常があります！");
            }

            layoutElem.Add(new XElement(ReseiverLayoutInfoTable.label,
                                new XElement(ReseiverLayoutInfoTable.label_Name, labelName),
                                labelElem.Element(ReseiverLayoutInfoTable.label_FontSize),
                                labelElem.Element(ReseiverLayoutInfoTable.label_FontFamily),
                                labelElem.Element(ReseiverLayoutInfoTable.label_FontColor),
                                labelElem.Element(ReseiverLayoutInfoTable.label_X),
                                labelElem.Element(ReseiverLayoutInfoTable.label_Y),
                                labelElem.Element(ReseiverLayoutInfoTable.label_Width),
                                labelElem.Element(ReseiverLayoutInfoTable.label_Height),
                                valueElem
                                )
                            );
        }
    }
}
