﻿using System;
using System.IO;
using Hitsuji.Framework.ViewModels;
using Hitsuji.Nengajou.Models.DataBases;
using Hitsuji.Nengajou.Views;

namespace Hitsuji.Nengajou.ViewModels
{
    public class MainWindowViewModel : AViewModel
    {
        public void initAppDataSourceSetting()
        {
            //各ディレクトリの存在確認・生成
            TableFactory.checkExistAndCreateDirectory(FilePathDataBase.getInfoDirectory());
            TableFactory.checkExistAndCreateDirectory(FilePathDataBase.getLayoutInfoDirectory());
            TableFactory.checkExistAndCreateDirectory(FilePathDataBase.getPrintLayoutDirectory());
            TableFactory.checkExistAndCreateDirectory(FilePathDataBase.getIllustDirectoryPath());

            //データソースファイルが存在しない場合は作成
            TableFactory.checkExistAndCreateTable(FilePathDataBase.getGroupFilePath(), TableFactory.createGroupInfoTable);
            TableFactory.checkExistAndCreateTable(FilePathDataBase.getYearFilePath(), TableFactory.createYearInfoTable);
            TableFactory.checkExistAndCreateTable(FilePathDataBase.getFontFamilyFilePath(), TableFactory.createFontFamilyInfoTable);
            TableFactory.checkExistAndCreateTable(FilePathDataBase.getMyPersonalFilePath(), TableFactory.createMyPersonalInfoTable);
            TableFactory.checkExistAndCreateTable(FilePathDataBase.getReseiverLayoutFilePath(), TableFactory.createReseiverLayoutInfoTable);
            TableFactory.checkExistAndCreateTable(FilePathDataBase.getIllustLayoutFilePath(), TableFactory.createIllustLayoutInfoTable);
            TableFactory.checkExistAndCreateTable(FilePathDataBase.getOtherPersonalFilePath(), TableFactory.createOtherPersonalInfoTable);

            TableFactory.checkExistAndCreateTable(FilePathDataBase.getIllustFileFilePath(), TableFactory.createDefaultIllust);
        }

        public void initViewModel()
        {
            //ビューモデルの準備
            //メインウィンドウロード時に表示されるビューに対しては、
            //メインウィンドウの各ビューオブジェクトを参照してビューモデルと関連付ける。
            var window1 = this.View as Window1;
            ViewModelManager.addViewModel(window1.MainAreaBaseV, new MainAreaBaseViewModel());
            ViewModelManager.addViewModel(window1.MainMenuV, new MainMenuViewModel());
            ViewModelManager.addViewModel(window1.PrimaryAreaBaseV, new PrimaryAreaBaseViewModel());
            ViewModelManager.addViewModel(window1.PrimaryMenuV, new PrimaryMenuViewModel());

            //動的に設定するビューはビューモデルとあらかじめ関連付けておいて、
            //必要な都度コレクションから取得可能な状態にしておく。
            ViewModelManager.addViewModel(new GroupInputView(), new GroupInputViewModel());
            ViewModelManager.addViewModel(new YearInputView(), new YearInputViewModel());
            ViewModelManager.addViewModel(new FontFamilyInputView(), new FontFamilyInputViewModel());
            ViewModelManager.addViewModel(new MyPersonalInputView(), new MyPersonalInputViewModel());
            ViewModelManager.addViewModel(new OtherPersonalInputView(), new OtherPersonalInputViewModel());
            ViewModelManager.addViewModel(new ReseiverLayoutView(), new ReseiverLayoutViewModel());
            ViewModelManager.addViewModel(new IllustLayoutView(), new IllustLayoutViewModel());
            //ViewModelManager.addViewModel(new PrimaryMenuView(), new PrimaryMenuViewModel());
            //ViewModelManager.addViewModel(new PrimaryAreaBaseView(), new PrimaryAreaBaseViewModel());
            ViewModelManager.addViewModel(new PrimaryListView(), new PrimaryListViewModel());
            //ViewModelManager.addViewModel(new MainMenuView(), new MainMenuViewModel());
            //ViewModelManager.addViewModel(new MainAreaBaseView(), new MainAreaBaseViewModel());
            ViewModelManager.addViewModel(new LayoutListView(), new LayoutListViewModel());
        }
    }
}
