﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using Hitsuji.Framework.ViewModels;
using Hitsuji.Nengajou.Models.Contents;
using Hitsuji.Nengajou.ObserverPattern.Observers;
using Hitsuji.Nengajou.ObserverPattern.Subjects;
using Hitsuji.Nengajou.Views;

namespace Hitsuji.Nengajou.ViewModels
{
    public class MyPersonalInputViewModel : AViewModel
    {

        public MyPersonalInputViewModel()
        {
            MyPersonalContentInfo_ = ContentModelFactory.createMyPersonalContentModel();
            (MyPersonalContentInfo_ as MyPersonalInfoContentModel).canExeDelegate_ = this.canExecuteMethod;
            (MyPersonalContentInfo_ as MyPersonalInfoContentModel).exeDelegate_ = this.executeMethod;

        }


        private MyPersonalInfoContentModel MyPersonalContentInfo_;
        public MyPersonalInfoContentModel MyPersonalContentInfo
        {
            get { return MyPersonalContentInfo_; }
            set { MyPersonalContentInfo_ = value; }
        }

        //コマンド canExecute
        protected bool canExecuteMethod()
        {
            var Model = MyPersonalContentInfo_ as MyPersonalInfoContentModel;

            if (string.IsNullOrEmpty(Model.Address1) &&
                string.IsNullOrEmpty(Model.Address2) &&
                string.IsNullOrEmpty(Model.Address3)) return false;

            if (string.IsNullOrEmpty(Model.Name1) &&
                string.IsNullOrEmpty(Model.Name2) &&
                string.IsNullOrEmpty(Model.Name3) &&
                string.IsNullOrEmpty(Model.Name4)) return false;
            
            return true;
        }


        //コマンド Execute
        protected void executeMethod()
        {
            try
            {
                var subject = new GetMyPersonalInfoSubject();
                var updateObserver = new UpdateMyPersonalDataSourceObserver();
                subject.models.Add(this.MyPersonalContentInfo.GetType().Name, this.MyPersonalContentInfo);
                subject.addObserver(updateObserver);
                subject.doMyRole();

                //強制的にビュー更新
                var primaryMenuVM = ViewModelManager.getInstance()["PrimaryMenuViewModel"] as PrimaryMenuViewModel;
                primaryMenuVM.onMyPersonalInfoMenuClicked();
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }

        //グループポップアップボタンクリック時
        public void onGroupPopUpButtonClicked(object sender, EventArgs e)
        {
            try
            {
                var popup = (this.View as MyPersonalInputView).GroupPopUp;
                if (popup.IsOpen == true)
                {
                    popup.IsOpen = false;
                    return;
                }

                var getGroupSubject = new GetGroupInfoSubject();
                var listboxInnerPopup = (this.View as MyPersonalInputView).listBoxGroupPopUp;

                getGroupSubject.doMyRole();
                listboxInnerPopup.Items.Clear();
                foreach (GroupInfoContentModel grp in getGroupSubject.models["infoList"] as List<IContentModel>)
                {
                    listboxInnerPopup.Items.Add(grp.Name);
                }

                popup.Width = 200;
                popup.Height = 200;
                popup.PlacementTarget = sender as UIElement;
                popup.IsOpen = true;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }
        //ポップアップ選択アイテム更新時
        public void onGroupSelectionItemChanged(object sender, SelectionChangedEventArgs e)
        {
            try
            {
                if ((e.Source as Selector).SelectedItem == null)
                    return;
                var view = (this.View as MyPersonalInputView);
                var getSubject = new GetGroupInfoSubject();
                getSubject.doMyRole();
                var list = getSubject.models["infoList"] as List<IContentModel>;

                //プロパティに反映
                foreach (GroupInfoContentModel grp in list)
                {
                    if ((e.Source as Selector).SelectedItem.ToString().Equals(grp.Name))
                    {
                        this.MyPersonalContentInfo.GroupKey = grp.Id;
                        break;
                    }
                }
                view.textBoxGroup.Text = (e.Source as Selector).SelectedItem.ToString();

                view.GroupPopUp.IsOpen = false;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }
    }
}
