﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Xml.Linq;
using Hitsuji.Framework.ViewModels;
using Hitsuji.Nengajou.Models.Contents;
using Hitsuji.Nengajou.ObserverPattern.Observers;
using Hitsuji.Nengajou.ObserverPattern.Subjects;
using Hitsuji.Nengajou.Views;

namespace Hitsuji.Nengajou.ViewModels
{
    public class OtherPersonalInputViewModel : AViewModel
    {

        public OtherPersonalInputViewModel()
        {
            OtherPersonalContentInfo = ContentModelFactory.createOtherPersonalContentModel();
            (OtherPersonalContentInfo as OtherPersonalInfoContentModel).canExeDelegate_ = this.canExecuteMethod;
            (OtherPersonalContentInfo as OtherPersonalInfoContentModel).exeDelegate_ = this.executeMethod;
        }

        public void initView()
        {
            var view = this.View as OtherPersonalInputView;
            
            //NPFポップアップ内のリスト初期化
            if (view.listBoxNPFInnerPopup.Items.Count <= 0)
            {
                var list = new List<string>();
                list.AddRange(new[] { "", "様", "様方", "殿", "さん", "さんへ", "君", "君へ", "ちゃん", "ちゃんへ", "へ" });
                foreach (string item in list)
                {
                    view.listBoxNPFInnerPopup.Items.Add(item);

                }
            }

        }

        public OtherPersonalInfoContentModel OtherPersonalContentInfo
        {
            get;
            set;
        }


        //コマンド canExecute
        protected bool canExecuteMethod()
        {
            if (string.IsNullOrEmpty(OtherPersonalContentInfo.Address1) &&
                string.IsNullOrEmpty(OtherPersonalContentInfo.Address2) &&
                string.IsNullOrEmpty(OtherPersonalContentInfo.Address3) 
                ||
                string.IsNullOrEmpty(OtherPersonalContentInfo.Name1) &&
                string.IsNullOrEmpty(OtherPersonalContentInfo.Name2) &&
                string.IsNullOrEmpty(OtherPersonalContentInfo.Name3) &&
                string.IsNullOrEmpty(OtherPersonalContentInfo.Name4)
                ||
                string.IsNullOrEmpty(OtherPersonalContentInfo.SenderKey)
                ||
                string.IsNullOrEmpty(OtherPersonalContentInfo.ReseiverLayoutKey))
                return false;

            return true;
        }


        //コマンド Execute
        protected void executeMethod()
        {
            try
            {
                var subject = new GetOtherPersonalInfoSubject();
                var updateObserver = new UpdateOtherPersonalDataSourceObserver();
                subject.models.Add(this.OtherPersonalContentInfo.GetType().Name, this.OtherPersonalContentInfo);
                subject.addObserver(updateObserver);
                subject.doMyRole();

                //強制的にビュー更新
                var primaryMenuVM = ViewModelManager.getInstance()["PrimaryMenuViewModel"] as PrimaryMenuViewModel;
                primaryMenuVM.onOtherPersonalMenuClicked();
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }


        
        //グループポップアップボタンクリック時
        public void onGroupPopUpButtonClicked(object sender, EventArgs e)
        {
            try
            {
                var popup = (this.View as OtherPersonalInputView).GroupPopUp;
                if (popup.IsOpen == true)
                {
                    popup.IsOpen = false;
                    return;
                }

                var getGroupSubject = new GetGroupInfoSubject();
                var listboxInnerPopup = (this.View as OtherPersonalInputView).listBoxGroupPopUp;

                getGroupSubject.doMyRole();
                listboxInnerPopup.Items.Clear();
                foreach (GroupInfoContentModel grp in getGroupSubject.models["infoList"] as List<IContentModel>)
                {
                    listboxInnerPopup.Items.Add(grp.Name);
                }

                popup.Width = 200;
                popup.Height = 200;
                popup.PlacementTarget = sender as UIElement;
                popup.IsOpen = true;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }
        //ポップアップ選択アイテム更新時
        public void onGroupSelectionItemChanged(object sender, SelectionChangedEventArgs e)
        {
            try
            {
                if ((e.Source as Selector).SelectedItem == null)
                    return;
                var view = (this.View as OtherPersonalInputView);
                var getSubject = new GetGroupInfoSubject();
                getSubject.doMyRole();
                var list = getSubject.models["infoList"] as List<IContentModel>;

                //プロパティに反映
                foreach (GroupInfoContentModel grp in list)
                {
                    if ((e.Source as Selector).SelectedItem.ToString().Equals(grp.Name))
                    {
                        this.OtherPersonalContentInfo.GroupKey = grp.Id;
                        break;
                    }
                }
                view.textBoxGroup.Text = (e.Source as Selector).SelectedItem.ToString();
                view.GroupPopUp.IsOpen = false;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }

        //NPFポップアップボタンクリック時
        private TextBox targetNPFTextBox;
        public void onNPFPopUpButtonClicked(object sender, EventArgs e, TextBox targetTextBox)
        {
            try
            {
                var popup = (this.View as OtherPersonalInputView).popupNPF;
                if (popup.IsOpen == true)
                {
                    popup.IsOpen = false;
                    return;
                }

                targetNPFTextBox = targetTextBox;
                popup.Width = 150;
                popup.Height = 200;
                popup.PlacementTarget = targetNPFTextBox;
                popup.IsOpen = true;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }
        //ポップアップ選択アイテム更新時
        public void onNPFSelectionItemChanged(object sender, SelectionChangedEventArgs e)
        {
            if ((e.Source as Selector).SelectedItem == null)
                return;
            var view = (this.View as OtherPersonalInputView);
            targetNPFTextBox.Text = (e.Source as Selector).SelectedItem.ToString();
            view.popupNPF.IsOpen = false;
            (e.Source as Selector).SelectedItem = null;
            targetNPFTextBox.Focus();
        }
        
        //送付者ポップアップボタンクリック時
        public void onSenderPopUpButtonClicked(object sender, EventArgs e)
        {
            try
            {
                var popup = (this.View as OtherPersonalInputView).popupSender;
                if (popup.IsOpen == true)
                {
                    popup.IsOpen = false;
                    return;
                }

                var getSubject = new GetMyPersonalInfoSubject();
                var listboxInnerPopup = (this.View as OtherPersonalInputView).listBoxSenderInnerPopup;
                getSubject.doMyRole();

                listboxInnerPopup.Items.Clear();
                foreach (MyPersonalInfoContentModel model in getSubject.models["infoList"] as List<IContentModel>)
                {
                    listboxInnerPopup.Items.Add(model.Id + ":" + model.Name1 + ".. (" + model.GroupName + ")");
                }

                popup.Width = 300;
                popup.Height = 200;
                popup.PlacementTarget = sender as UIElement;
                popup.IsOpen = true;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }
        //ポップアップ選択アイテム更新時
        public void onSenderSelectionItemChanged(object sender, SelectionChangedEventArgs e)
        {
            try
            {
                if ((e.Source as Selector).SelectedItem == null)
                    return;
                var view = (this.View as OtherPersonalInputView);
                var getSubject = new GetMyPersonalInfoSubject();
                getSubject.doMyRole();

                //プロパティに反映
                foreach (MyPersonalInfoContentModel model in getSubject.models["infoList"] as List<IContentModel>)
                {
                    if ((e.Source as Selector).SelectedItem.ToString().Equals(model.Id + ":" + model.Name1 + ".. (" + model.GroupName + ")"))
                    {
                        this.OtherPersonalContentInfo.SenderKey = model.Id;
                        break;
                    }
                }
                view.popupSender.IsOpen = false;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }

        //レイアウトポップアップボタンクリック時
        public void onReseiverLayoutPopUpButtonClicked(object sender, EventArgs e)
        {
            try
            {
                var popup = (this.View as OtherPersonalInputView).popupReseiverLayout;
                if (popup.IsOpen == true)
                {
                    popup.IsOpen = false;
                    return;
                }

                var getSubject = new GetReseiverLayoutInfoSubject();
                var listboxInnerPopup = (this.View as OtherPersonalInputView).listBoxReseiverLayoutInnerPopup;

                getSubject.doMyRole();
                listboxInnerPopup.Items.Clear();
                foreach (XElement elem in getSubject.LayoutElements)
                {
                    listboxInnerPopup.Items.Add(elem.Element("ID").Value + "：" + elem.Element("NAME").Value);
                }

                popup.Width = 300;
                popup.Height = 200;
                popup.PlacementTarget = sender as UIElement;
                popup.IsOpen = true;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }
        //ポップアップ選択アイテム更新時
        public void onReseiverLayoutSelectionItemChanged(object sender, SelectionChangedEventArgs e)
        {
            try
            {
                if ((e.Source as Selector).SelectedItem == null)
                    return;
                var view = (this.View as OtherPersonalInputView);
                var getSubject = new GetReseiverLayoutInfoSubject();
                getSubject.doMyRole();
                var list = getSubject.LayoutElements;

                //プロパティに反映
                foreach (XElement elem in list)
                {
                    if ((e.Source as Selector).SelectedItem.ToString().Equals(elem.Element("ID").Value + "：" + elem.Element("NAME").Value))
                    {
                        this.OtherPersonalContentInfo.ReseiverLayoutKey = elem.Element("ID").Value;
                        break;
                    }
                }
                view.popupReseiverLayout.IsOpen = false;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }
    }
}
